from PyQt4.QtGui import *

from ui_mainwindow import Ui_MainWindow
import utils


class MainWindow(QDialog):
    def __init__(self):
        QDialog.__init__(self)

        self.invoice = None
        self.email = None

        self.ui = Ui_MainWindow()
        self.ui.setupUi(self)

        self.set_loginButton_enabled()

        # Handle the list
        self.model = QStandardItemModel(self.ui.listView)
        self.ui.listView.setModel(self.model)

        data = utils.get_configs()
        self.fill_list(data)

    def on_loginButton_released(self):
        invoice = self.ui.invoiceEdit.text()
        email = self.ui.emailEdit.text()

        if not email or not invoice:
            return

        data = utils.get_configs(email=email, invoice=invoice)
        if not data:
            return

        if data['valid'] == 1:
            self.invoice = invoice
            self.email = email
        else:
            self.invoice = self.email = None

        self.fill_list(data)

    def set_loginButton_enabled(self):
        enabled = True
        if not self.ui.emailEdit.text() or not self.ui.invoiceEdit.text():
            enabled = False
        self.ui.loginButton.setEnabled(enabled)

    def on_invoiceEdit_textChanged(self, text):
        self.set_loginButton_enabled()

    def on_emailEdit_textChanged(self, text):
        self.set_loginButton_enabled()

    def fill_list(self, data):
        self.model.clear()
        self.ui.descriptionLabel.setText("")
        self.ui.applyButton.setEnabled(False)
        for item in data['sources']:
            if item.get('type', 0) != 1:
                 # not a config, ignore
                continue

            modelItem = QStandardItem()
            # FIXME: subclass QStandardItem
            modelItem.data = item
            modelItem.setText(item['desc'])
            self.model.appendRow(modelItem)

    def on_listView_clicked(self, index):
        item = self.model.itemFromIndex(index)
        desc = item.data['longdesc']
        self.ui.descriptionLabel.setText(desc)
        self.ui.applyButton.setEnabled(True)

    def on_applyButton_released(self):
        try:
            index = self.ui.listView.selectedIndexes()[0]
        except IndexError:
            return

        item = self.model.itemFromIndex(index)

        self.msg = QMessageBox(QMessageBox.Information, "Installing",
                               "Installation in progres...")
        self.msg.setStandardButtons(QMessageBox.NoButton)

        self.installer = utils.Installer(item.data['version'],
                                         self.email,
                                         self.invoice)
        self.installer.downloadFinished.connect(self.onDownloadFinished)
        self.installer.installFinished.connect(self.onInstallFinished)
        self.installer.start()

        self.msg.show()

    def onDownloadFinished(self, ok):
        if not ok:
            del self.msg
            self.msg = QMessageBox.critical(None, "Error",
                        "Impossible to download the config.")

    def onInstallFinished(self, ok, text):
        del self.msg
        if not ok:
            self.msg = QMessageBox.critical(None, "Error",
                        "An error occured. Try again later.")
        else:
            self.msg = QMessageBox.information(None, "Success", text)

