import json
import os
import subprocess

from PyQt4.QtCore import QThread, pyqtSignal
import requests


def sanitize_json(data):
    data = data.replace('"', r'\"').replace("'", '"')
    data = data.replace('\n', '\\n').replace('\r', '')

    return data


def get_configs(email=None, invoice=None):
    url = "http://www.linutop.com/shop/validate_account.php?configs=1"
    if email is not None:
        url += "&email=%s" % email
    if invoice is not None:
        url += "&invoice=%s" % invoice

    headers = {'X-Linutop-Arch': os.uname()[4]}
    r = requests.get(url, headers=headers)
    if r.status_code != 200:
        return None

    return json.loads(sanitize_json(r.text))


class Installer(QThread):
    downloadFinished = pyqtSignal(bool)
    installFinished = pyqtSignal(bool, str)

    def __init__(self, script_id, email, invoice):
        super(Installer, self).__init__()
        self.script_id = script_id
        self.email = email
        self.invoice = invoice
        self.script_archive = None
        self.script_dir = None

    def download_config(self):
        url = ("http://configs.linutop.com/getconfig.php?version=%s" %
               self.script_id)
        if self.email is not None:
            url += "&email=%s" % self.email
        if self.invoice is not None:
            url += "&invoice=%s" % self.invoice

        headers = {'X-Linutop-Arch': os.uname()[4]}
        r = requests.get(url, headers=headers)
        if r.status_code != 200:
            self.downloadFinished.emit(False)
            return

        name = '%s.tar.gz' % self.script_id
        path = '/tmp/%s'% name
        with open(path, 'w') as fd:
            for chunk in r.iter_content(1024):
                fd.write(chunk)

        self.script_archive = path
        self.script_dir = '/tmp/%s' % self.script_id
        self.downloadFinished.emit(True)

    def install_config(self):
        if self.script_archive is None:
            self.installFinished(False, "")
            return 

        if not os.path.isdir(self.script_dir):
            os.mkdir(self.script_dir)

        cmd = "tar -zx -f %s -C %s" % (self.script_archive, self.script_dir)
        try:
            subprocess.check_call(cmd, shell=True)
        except subprocess.CalledProcessError:
            self.installFinished.emit(False, "")

        cmd = '%s/run.sh' % self.script_dir
        os.chmod(cmd, 0700)
        try:
            output = subprocess.check_output(cmd, shell=True)
        except subprocess.CalledProcessError:
            self.installFinished.emit(False, "")

        self.installFinished.emit(True, output)

    def run(self):
        self.download_config()
        if self.script_archive is not None:
            self.install_config()
